@@ -1,4 +1,4 @@
-/* $Id: RgX.c 7838 2006-04-08 12:11:17Z kb $
+/* $Id: RgX.c 11396 2008-12-08 11:04:43Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -897,7 +897,7 @@ RgXQX_divrem(GEN x, GEN y, GEN T, GEN *pr)
     p1 = gel(x,i);
     for (j=0; j<=i && j<=dz; j++)
       p1 = gsub(p1, gmul(gel(z,j),gel(y,i-j)));
-    tetpil=avma; p1 = grem(p1, T); if (signe(p1)) { sx = 1; break; }
+    tetpil=avma; p1 = grem(p1, T); if (!gcmp0(p1)) { sx = 1; break; }
     if (!i) break;
     avma=av;
   }
