@@ -1,4 +1,4 @@
-/* $Id: alglin2.c 9661 2008-02-18 10:07:27Z kb $
+/* $Id: alglin2.c 11336 2008-11-22 17:33:45Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -3450,7 +3450,7 @@ static GEN
 gbezout_step(GEN *pa, GEN *pb, GEN *pu, GEN *pv)
 {
   GEN a = *pa, b = *pb, d;
-  if (!signe(a))
+  if (gcmp0(a))
   {
     *pa = gen_0; *pu = gen_0;
     *pb = gen_1; *pv = gen_1; return b;
