@@ -1,4 +1,4 @@
-/* $Id: arith1.c 10290 2008-06-10 15:09:08Z kb $
+/* $Id: arith1.c 12104 2010-02-03 00:15:18Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -289,8 +289,12 @@ gener(GEN m)
   }
   if (e == 2) /* m = 0 mod 2 */
   {
-    GEN q = shifti(m,-1); x = (GEN) gener(q)[2];
-    if (!mod2(x)) x = addii(x,q);
+    if (equaliu(m,2)) x = gen_1; 
+    else
+    {
+      GEN q = shifti(m,-1); x = gel(gener(q),2);
+      if (!mod2(x)) x = addii(x,q);
+    }
     gel(z,2) = gerepileuptoint(av, x); return z;
   }
 
