@@ -1,4 +1,4 @@
-/* $Id: buch1.c 10279 2008-06-09 07:12:47Z kb $
+/* $Id: buch1.c 12064 2010-01-09 18:30:42Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -73,7 +73,7 @@ check_pq(GEN gp, GEN z, long d, GEN D)
 {
   ulong p = itou(gp);
   if (!umodiu(z,p) || kross(d,(long)p) <= 0 || 
-    gcmp1((GEN)redimag(primeform_u(D,p))[1]));
+    gcmp1((GEN)redimag(primeform_u(D,p))[1]))
       pari_err(talker,"[quadhilbert] incorrect values in pq: %lu", p);
   return p;
 }
@@ -1535,9 +1535,9 @@ buchquad(GEN D, double cbach, double cbach2, long RELSUP, long prec)
   {
     if (cmpiu(Disc,4) <= 0)
     {
-      GEN z = cgetg(6,t_VEC);
-      gel(z,1) = gel(z,4) = gel(z,5) = gen_1;
-      gel(z,2) = gel(z,3) = cgetg(1,t_VEC); return z;
+      GEN z = cgetg(5,t_VEC);
+      gel(z,1) = gel(z,4) = gen_1; gel(z,2) = gel(z,3) = cgetg(1,t_VEC);
+      return z;
     }
     PRECREG = 0;
   } else {
