@@ -1,4 +1,4 @@
-/* $Id: gen1.c 10289 2008-06-10 15:05:56Z kb $
+/* $Id: gen1.c 11470 2008-12-19 19:33:27Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1487,11 +1487,10 @@ gmul(GEN x, GEN y)
         {
           case t_VEC:
             z = cgetg(ly,t_MAT);
-            for (i=1; i<ly; i++)
+            for (j=1; j<ly; j++)
             {
-              p1 = gmul(gel(y,i),x);
-              if (typ(p1) != t_COL) pari_err(operi,"*",x,y);
-              gel(z,i) = p1;
+              GEN c = cgetg(lx, t_COL); gel(z,j) = c;
+              for (i=1; i<lx; i++) gel(c,i) = gmul(gel(x,i), gel(y,j));
             }
             return z;
 
@@ -1508,9 +1507,7 @@ gmul(GEN x, GEN y)
         {
           case t_VEC:
             if (lx != 2) pari_err(operi,"*",x,y);
-            z = cgetg(ly,t_MAT);
-            for (i=1; i<ly; i++) gel(z,i) = gmul(gel(y,i),gel(x,1));
-            return z;
+            return gmul(gel(x,1), y);
 
           case t_COL:
             if (lx != ly) pari_err(operi,"*",x,y);
