@@ -1,4 +1,4 @@
-/* $Id: gen2.c,v 1.154.2.1 2006/09/09 18:07:49 bill Exp $
+/* $Id: gen2.c 10277 2008-06-09 06:42:35Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -822,7 +822,11 @@ Z_lvalrem_stop(GEN n, ulong p, int *stop)
     {
       do { 
         v++; N = q;
-        if (v == 32) { v = 32 + Z_pvalrem_DC(N, utoipos(p), &N); break; }
+        if (v == 16) {
+          v += Z_pvalrem_DC(N, sqru(p), &N) << 1;
+          q = diviu_rem(N, p, &r); if (!r) { v++; N = q; }
+          break;
+        }
         q = diviu_rem(N, p, &r);
       } while (!r);
       affii(N, n);
