@@ -1,4 +1,4 @@
-/* $Id: gen3.c 9124 2007-10-29 10:53:06Z kb $
+/* $Id: gen3.c 11395 2008-12-08 10:46:23Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1664,12 +1664,14 @@ integ(GEN x, long v)
   {
     case t_POL:
       vx = varn(x); lx = lg(x);
-      if (varncmp(vx, v) < 0) v = vx;
-      if (lx == 2) return zeropol(v);
+      if (lx == 2) {
+        if (varncmp(vx, v) < 0) v = vx;
+        return zeropol(v);
+      }
       if (varncmp(vx, v) > 0)
       {
         y = cgetg(4,t_POL);
-	y[1] = x[1];
+	y[1] = evalsigne(1) | evalvarn(v);
         gel(y,2) = gen_0;
         gel(y,3) = gcopy(x); return y;
       }
