@@ -1,4 +1,4 @@
-/* $Id: polarit1.c 9299 2007-12-02 11:20:18Z kb $
+/* $Id: polarit1.c 11851 2009-07-21 21:52:51Z bill $
 
 Copyright (C) 2000-2004  The PARI group.
 
@@ -260,7 +260,8 @@ FpX_quad_root(GEN x, GEN p, int unknown)
   if (unknown && kronecker(D,p) == -1) return NULL;
 
   s = Fp_sqrt(D,p);
-  if (!s) pari_err(talker,"not a prime in FpX_quad_root");
+  /* p is not prime, go on and give e.g. maxord a chance to recover */
+  if (!s) return NULL;
   u = addis(shifti(p,-1), 1); /* = 1/2 */
   return modii(mulii(u, subii(s,b)), p);
 }
