@@ -1,4 +1,4 @@
-/* $Id: trans1.c 9117 2007-10-29 10:26:09Z kb $
+/* $Id: trans1.c 12114 2010-02-03 22:59:09Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -382,7 +382,7 @@ upowuu(ulong p, ulong k)
   ulong i, pk;
 
   if (!k) return 1;
-  if (p == 2) return 1<<k;
+  if (p == 2) return 1UL<<k;
   pk = p; for (i=2; i<=k; i++) pk *= p;
   return pk;
 }
@@ -1266,9 +1266,8 @@ mpexp(GEN x)
   GEN a, z;
 
   if (!sx) {
-    l = 3 + ((-expo(x)) >> TWOPOTBITS_IN_LONG);
-    if (l < 3) l = 3;
-    return real_1(l);
+    long e = expo(x);
+    return e >= 0? real_0_bit(e): real_1(nbits2prec(-e));
   }
 
   l = lg(x);
@@ -2121,7 +2120,7 @@ mpsincos(GEN x, GEN *s, GEN *c)
   {
     long e = expo(x);
     *s = real_0_bit(e);
-    *c = real_1(3 + ((-e) >> TWOPOTBITS_IN_LONG));
+    *c = e >= 0? real_0_bit(e): real_1(nbits2prec(-e));
     return;
   }
 
