@@ -1,4 +1,4 @@
-/* $Id: trans2.c,v 1.123.2.2 2006/12/11 14:24:41 kb Exp $
+/* $Id: trans2.c 10281 2008-06-09 11:10:14Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -198,7 +198,14 @@ gasin(GEN x, long prec)
       if (gcmp0(y)) return gcopy(y);
       /* lg(y) > 2*/
       if (valp(y) < 0) pari_err(negexper,"gasin");
-      p1 = gdiv(derivser(y), gsqrt(gsubsg(1,gsqr(y)),prec));
+      p1 = gsubsg(1,gsqr(y));
+      if (gcmp0(p1))
+      {
+        GEN t = Pi2n(-1,prec);
+        if (gsigne(gel(y,2)) < 0) setsigne(t, -1);
+        return gerepileupto(av, scalarser(t, varn(y), valp(p1)>>1));
+      }
+      p1 = gdiv(derivser(y), gsqrt(p1,prec));
       a = integ(p1,varn(y));
       if (!valp(y)) a = gadd(a, gasin(gel(y,2),prec));
       return gerepileupto(av, a);
@@ -263,8 +270,9 @@ gacos(GEN x, long prec)
       if (valp(y) < 0) pari_err(negexper,"gacos");
       if (lg(y) > 2)
       {
-	p1 = integ(gdiv(gneg(derivser(y)),
-                        gsqrt(gsubsg(1,gsqr(y)),prec)), varn(y));
+	p1 = gsubsg(1,gsqr(y));
+	if (gcmp0(p1)) return zeroser(varn(y), valp(p1)>>1);
+	p1 = integ(gdiv(gneg(derivser(y)), gsqrt(p1,prec)), varn(y));
 	if (gcmp1(gel(y,2)) && !valp(y)) /*y = 1+O(y^k), k>=1*/
 	  return gerepileupto(av, p1);
       }
@@ -546,8 +554,14 @@ gash(GEN x, long prec)
       av = avma; if (!(y = toser_i(x))) break;
       if (gcmp0(y)) return gcopy(y);
       if (valp(y) < 0) pari_err(negexper,"gash");
-
-      p1 = gdiv(derivser(y), gsqrt(gaddsg(1,gsqr(y)),prec));
+      p1 = gaddsg(1,gsqr(y));
+      if (gcmp0(p1))
+      {
+        GEN t = PiI2n(-1,prec);
+        if ( gsigne(imag_i(gel(y,2))) < 0 ) setsigne(gel(t,2), -1);
+        return gerepileupto(av, scalarser(t, varn(y), valp(p1)>>1));
+      }
+      p1 = gdiv(derivser(y), gsqrt(p1,prec));
       a = integ(p1,varn(y));
       if (!valp(y)) a = gadd(a, gash(gel(y,2),prec));
       return gerepileupto(av, a);
@@ -609,7 +623,9 @@ gach(GEN x, long prec)
         if (!v) return gcopy(y);
         return gerepileupto(av, gadd(y, PiI2n(-1, prec)));
       }
-      p1 = gdiv(derivser(y), gsqrt(gsubgs(gsqr(y),1),prec));
+      p1 = gsubgs(gsqr(y),1);
+      if (gcmp0(p1)) { avma = av; return zeroser(varn(y), valp(p1)>>1); }
+      p1 = gdiv(derivser(y), gsqrt(p1,prec));
       a = integ(p1, varn(y));
       if (v)
         p1 = PiI2n(-1, prec); /* I Pi/2 */
@@ -923,6 +939,7 @@ trans_fix_arg(long *prec, GEN *s0, GEN *sig, pari_sp *av, GEN *res)
   if (typ(*s0)==t_COMPLEX && gcmp0(gel(*s0,2))) *s0 = gel(*s0,1);
   s = *s0;
   l = precision(s); if (!l) l = *prec;
+  if (l < 3) l = 3;
 
   if (typ(s) == t_COMPLEX)
   { /* s = sig + i t */
