@@ -1,4 +1,4 @@
-/* $Id: gp_rl.c 7394 2005-11-09 18:21:51Z kb $
+/* $Id: gp_rl.c 12098 2010-01-29 13:40:22Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -230,9 +230,10 @@ pari_rl_forward_sexp(int count, int key)
     if (is_keyword_char(rl_line_buffer[rl_point]))
     {
       while ( is_keyword_char(rl_line_buffer[rl_point])
-              && (!((dir == 1) ? (rl_point >= rl_end) : (rl_point <= 0))
+              && (!((dir == 1) ? (rl_point >= rl_end) : (rl_point <= 0 && (lfail = 1)))
                   || (move_point = 0)))
         rl_point += dir;
+      if (deep && lfail) goto fail;
       if (!deep) count--;
     }
     else if (strchr(paropen,rl_line_buffer[rl_point]))
@@ -800,7 +801,7 @@ print_escape_string(char *s)
       case '"': *t++ = '\\'; continue;
     }
   *t++ = '"';
-  *t++ = 0; printf(t0); free(t0);
+  *t = '\0'; puts(t0); free(t0);
 }
 
 static char *
