@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/gmp/mp.c"
-/* $Id: mp.c,v 1.87.2.4 2007-12-05 09:52:56 bill Exp $
+/* $Id: mp.c 10295 2008-06-10 16:03:56Z kb $
 
 Copyright (C) 2002-2003  The PARI group.
 
@@ -294,7 +294,7 @@ affir(GEN x, GEN y)
     }
     for (i=2; i<ly; i++,xd=int_precW(xd)) y[i]=*xd;
     /* lx > ly: round properly */
-    if (x[2+lx-ly] & HIGHBIT) roundr_up_ip(y, ly);
+    if (x[lx-ly+1] & HIGHBIT) roundr_up_ip(y, ly);
   }
 }
 
