@@ -1,5 +1,5 @@
-#line 2 "../src/kernel/hppa64/level0.h"
-/* $Id: asm0.h 7881 2006-04-18 17:23:00Z kb $
+#line 2 "../src/kernel/hppa64/asm0.h"
+/* $Id: asm0.h 11631 2009-02-28 22:03:38Z bill $
 
 Copyright (C) 2004  The PARI group.
 
