@@ -1,5 +1,5 @@
-#line 2 "../src/kernel/ix86/level0.h"
-/* $Id: asm0.h 9123 2007-10-29 10:47:21Z kb $
+#line 2 "../src/kernel/ix86/asm0.h"
+/* $Id: asm0.h 11631 2009-02-28 22:03:38Z bill $
 
 Copyright (C) 2000  The PARI group.
 
