@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp_indep.c"
-/* $Id: mp_indep.c 10294 2008-06-10 15:58:51Z kb $
+/* $Id: mp_indep.c 12097 2010-01-29 11:57:49Z bill $
 
 Copyright (C) 2000  The PARI group.
 
@@ -605,7 +605,7 @@ usqrtsafe(ulong a)
 {
   ulong x = (ulong)sqrt((double)a);
 #ifdef LONG_IS_64BIT
-  ulong y = x+1; if (y <= MAXHALFULONG && y*y <= a) x = y;
+  if (x > LOWMASK || x*x > a) x--;
 #endif
   return x;
 }
