@@ -1,4 +1,4 @@
-/* $Id: anal.c,v 1.249 2006/04/11 15:03:44 kb Exp $
+/* $Id: anal.c 10414 2008-07-08 15:55:11Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -562,7 +562,7 @@ typedef struct var_cell {
 } var_cell;
 #define INITIAL NULL
 enum {PUSH_VAL = 0, COPY_VAL = 1};
-#define killvalue(v) pop_val(get_ep(v))
+#define killvalue(v) pop_val_full(get_ep(v))
 /* Push x on value stack associated to ep. Assume EpVALENCE(ep)=EpVAR/EpGVAR */
 static void
 new_val_cell(entree *ep, GEN x, char flag)
@@ -635,11 +635,23 @@ push_val(entree *ep, GEN a) { new_val_cell(ep, a, PUSH_VAL); }
 
 /* kill ep->value and replace by preceding one, poped from value stack */
 void
+pop_val_full(entree *ep)
+{
+  var_cell *v = (var_cell*) ep->args;
+
+  if (v == INITIAL) return;
+  killbloc((GEN)ep->value); /* inspect components to avoid memory leaks */
+  ep->value = v->value;
+  ep->args  = (void*) v->prev;
+  free((void*)v);
+}
+void
 pop_val(entree *ep)
 {
   var_cell *v = (var_cell*) ep->args;
 
   if (v == INITIAL) return;
+  /* otherwise often not a valid GEN due to GC */
   if (v->flag == COPY_VAL) killbloc((GEN)ep->value);
   ep->value = v->value;
   ep->args  = (void*) v->prev;
@@ -2183,7 +2195,7 @@ identifier(void)
 	m = ((long (*)(ANYARG))call)(_ARGS_);
 	res = stoi(m); break;
 
-      case RET_VOID:
+      default: /* RET_VOID */
 	((void (*)(ANYARG))call)(_ARGS_);
 	res = gnil; break;
       }
