@@ -1,4 +1,4 @@
-/* $Id: init.c,v 1.305.2.2 2007-10-29 10:19:39 kb Exp $
+/* $Id: init.c 9689 2008-02-23 06:41:39Z kb $
 
 Copyright (C) 2000-2003  The PARI group.
 
@@ -1977,6 +1977,19 @@ msgtimer(char *format, ...)
 /*                   FUNCTIONS KNOWN TO THE ANALYZER               */
 /*                                                                 */
 /*******************************************************************/
+GEN
+pari_version(void) {
+  GEN v = cgetg(4, t_VEC);
+  const ulong mask = (1<<PARI_VERSION_SHIFT) - 1;
+  ulong major, minor, patch, n = PARI_VERSION_CODE;
+  patch = n & mask; n >>= PARI_VERSION_SHIFT;
+  minor = n & mask; n >>= PARI_VERSION_SHIFT;
+  major = n;
+  gel(v,1) = utoi(major);
+  gel(v,2) = utoi(minor);
+  gel(v,3) = utoi(patch); return v;
+}
+
 GEN
 geni(void) { return gi; }
 
