@@ -1,4 +1,4 @@
-/* $Id: nffactor.c 10291 2008-06-10 15:50:00Z kb $
+/* $Id: nffactor.c 11784 2009-06-26 09:38:07Z bill $
 
 Copyright (C) 2000-2004  The PARI group.
 
@@ -934,9 +934,14 @@ NEXT:
 END:
   if (degpol(pol) > 0)
   { /* leftover factor */
-    if (signe(leading_term(pol)) < 0) pol = gneg_i(pol);
+    GEN LC = leading_term(pol);
+    if (signe(LC) < 0) { pol = gneg_i(pol); LC = leading_term(pol); }
 
-    if (C2lt && lfamod < 2*K) pol = QXQX_normalize(Q_primpart(pol), nfpol);
+    if (lfamod >= 2*K) {
+      if (!equalii(LC, lc)) pol = RgX_Rg_mul(pol, gdiv(lc,LC));
+    } else {
+      if (C2lt) pol = QXQX_normalize(Q_primpart(pol), nfpol);
+    }
     setlg(famod, lfamod+1);
     gel(listmod,cnt) = shallowcopy(famod);
     gel(fa,cnt++) = pol;
@@ -1243,7 +1248,9 @@ nf_LLL_cmbf(nfcmbf_t *T, GEN p, long k, long rec)
       h = gel(z,tnew+1);
       /* make Newton sums integral */
       lPpow = mul_content(lPpow, dn);
-      if (lPpow) h = FpX_red(gmul(h,lPpow), pk);
+      if (lPpow)
+        h = (typ(h) == t_INT)? modii(mulii(h, lPpow), L->pk): 
+                               FpX_Fp_mul(h, lPpow, L->pk);
       gel(Tra,i) = nf_bestlift(h, NULL, L); /* S_tnew(famod) */
     }
 
@@ -1409,7 +1416,7 @@ nf_pick_prime(long ct, GEN nf, GEN polbase, long fl,
   *lt  = leading_term(polbase); /* t_INT */
   if (gcmp1(*lt)) *lt = NULL;
   dk = absi(gel(nf,3));
-  bad = mulii(dk,gel(nf,4)); if (*lt) bad = mulii(bad, *lt);
+  bad = mulii(dk,gel(nf,4));
 
   /* FIXME: slow factorization of large polynomials over large Fq */
   maxf = 1;
@@ -1428,6 +1435,7 @@ nf_pick_prime(long ct, GEN nf, GEN polbase, long fl,
   {
     GEN aT, apr, ap, modpr, red;
     long anbf;
+    ulong ltp = 0;
     pari_timer ti_pr;
 
     GEN list, r = NULL, fa = NULL;
@@ -1437,6 +1445,7 @@ nf_pick_prime(long ct, GEN nf, GEN polbase, long fl,
     {
       NEXT_PRIME_VIADIFF_CHECK(pp, pt);
       if (! umodiu(bad,pp)) continue;
+      if (*lt) { ltp = umodiu(*lt, pp); if (!ltp) continue; }
       ap = utoipos(pp);
       list = (GEN)FpX_factor(nfpol, ap)[1];
       if (maxf == 1)
@@ -1463,12 +1472,14 @@ nf_pick_prime(long ct, GEN nf, GEN polbase, long fl,
     if (!aT)
     { /* degree 1 */
       red = ZX_to_Flx(red, pp);
+      if (ltp) red = Flx_normalize(red, pp);
       if (!Flx_is_squarefree(red, pp)) { avma = av2; continue; }
       anbf = fl? Flx_nbroots(red, pp): Flx_nbfact(red, pp);
     }
     else
     {
       GEN q;
+      if (ltp) red = FqX_normalize(red, aT,ap);
       if (!FqX_is_squarefree(red,aT,ap)) { avma = av2; continue; }
       q = gpowgs(ap, degpol(aT));
       anbf = fl? FqX_split_deg1(&fa, red, q, aT, ap)
