@@ -1,4 +1,4 @@
-/* $Id: thue.c 10292 2008-06-10 15:55:18Z kb $
+/* $Id: thue.c 11397 2008-12-08 17:08:13Z kb $
 
 Copyright (C) 2000  The PARI group.
 
@@ -1239,24 +1239,28 @@ GEN
 bnfisintnorm(GEN bnf, GEN a)
 {
   pari_sp av = avma;
-  GEN unit = NULL, z, x, res = bnfisintnormabs(bnf, a), nf = checknf(bnf);
-  long sNx, i, j = 1, l = lg(res), sa = signe(a);
+  GEN unit = NULL, z = bnfisintnormabs(bnf, a);
+  GEN nf = checknf(bnf), T = gel(nf,1);
+  long sNx, i, j, N = degpol(T), l = lg(z), sa = signe(a);
   long norm_1 = 0; /* gcc -Wall */
 
-  z = cgetg(l, t_VEC);
-  for (i=1; i<l; i++)
+  for (i = j = 1; i<l; i++)
   {
-    x = gel(res,i);
-    sNx = signe(subres(gel(nf,1), x));
+    GEN x = gel(z,i);
+    int xpol = (typ(x) == t_POL);
+
+    if (xpol) sNx = signe(ZX_resultant(T, Q_primpart(x)));
+    else      sNx = gsigne(x) < 0 && odd(N) ? -1 : 1;
     if (sNx != sa)
     {
       if (! unit) norm_1 = get_unit_1(bnf, &unit);
-      if (norm_1) x = gmul(unit,x);
-      else
+      if (!norm_1)
       {
         if (DEBUGLEVEL > 2) fprintferr("%Z eliminated because of sign\n",x);
         continue;
       }
+      if (xpol) x = (unit == gen_m1)? RgX_neg(x): RgXQ_mul(unit,x,T);
+      else      x = (unit == gen_m1)? gneg(x): RgX_Rg_mul(unit,x);
     }
     gel(z,j++) = x;
   }
